// Student: Dragoi Marius-Ninel Catalin
// Grupa: 333CC

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "lcd.h"

#define USE_ACCELEROMETER 0

#define XCHANNEL 0
#define YCHANNEL 1
#define ZCHANNEL 2
uint8_t channel;

#define THRESHOLD		15
#define PAD_INCREMENT	2

#define DEBOUNCE_TIME 100

#define ACCY_0G			317

unsigned int entropy = 0;

typedef enum { RIGHT = PC0, LEFT = PC1} Button;

typedef struct {
	int x1;
	int x2;
} gap;

gap l1_gaps[6];
gap l2_gaps[6];
gap l3_gaps[6];

static int level = 1;

void draw_world() {
	l1_gaps[0].x1 = 55;
	l1_gaps[0].x2 = 65;
	
	l1_gaps[1].x1 = 14;
	l1_gaps[1].x2 = 21;
	
	l1_gaps[2].x1 = 58;
	l1_gaps[2].x2 = 68;
	
	l1_gaps[3].x1 = 13;
	l1_gaps[3].x2 = 20;
	
	l1_gaps[4].x1 = 70;
	l1_gaps[4].x2 = 78;
	
	l1_gaps[5].x1 = 30;
	l1_gaps[5].x2 = 40;
	
	
	
	
	l2_gaps[0].x1 = 30;
	l2_gaps[0].x2 = 35;
	
	l2_gaps[1].x1 = 40;
	l2_gaps[1].x2 = 45;
	
	l2_gaps[2].x1 = 50;
	l2_gaps[2].x2 = 55;
	
	l2_gaps[3].x1 = 12;
	l2_gaps[3].x2 = 17;
	
	l2_gaps[4].x1 = 70;
	l2_gaps[4].x2 = 75;
	
	l2_gaps[5].x1 = 0;
	l2_gaps[5].x2 = 5;
	
	
	
	
	l3_gaps[0].x1 = 70;
	l3_gaps[0].x2 = 75;
	
	l3_gaps[1].x1 = 5;
	l3_gaps[1].x2 = 10;
	
	l3_gaps[2].x1 = 50;
	l3_gaps[2].x2 = 57;
	
	l3_gaps[3].x1 = 31;
	l3_gaps[3].x2 = 38;
	
	l3_gaps[4].x1 = 70;
	l3_gaps[4].x2 = 80;
	
	l3_gaps[5].x1 = 11;
	l3_gaps[5].x2 = 15;
	
}

int key_down(Button button)
{
    entropy += button;
    
    if (bit_is_clear(PINC, button))
    {
        _delay_ms(DEBOUNCE_TIME);
        
        if (bit_is_clear(PINC, button))
            return 1;
    }

    return 0;
}

int pressed_button(Button btn)
{
    return (PINC & (1 << btn)) == 0 ? 1 : 0;
}

int ADC_get(uint8_t channel){

	ADMUX = (ADMUX & ~(0x1f << MUX0)) | channel;

	ADCSRA |= (1 << ADSC);
	while(ADCSRA & (1 << ADSC));
	
	return ADC;
}

void ADC_init(void){
	ADMUX = (1 << REFS0);
	ADCSRA = (1 << ADEN) | (7 << ADPS0);
}


unsigned int frame;
static unsigned char lcd_buffer[LCD_Y_RES / 8][LCD_X_RES];

unsigned int score = 0;

static int xball = 3, yball = 3;

void clear_lcd_buffer(void)
{
    for(int i = 0; i < LCD_Y_RES / 8; i++)
        for(int j = 0; j < LCD_X_RES; j++)
            lcd_buffer[i][j] = 0;
}

void flush_lcd_buffer(void)
{
    lcd_base_addr(0);
    for(int i = 0; i < LCD_Y_RES / 8; i++)
        for(int j = 0; j < LCD_X_RES; j++)
                lcd_send(lcd_buffer[i][j], LCD_DATA);
                
    clear_lcd_buffer();
}

void put_pixel(unsigned char row, unsigned char col)
{
    lcd_buffer[row / 8][col] |= 1 << (row % 8) ;
}

void clear_pixel(unsigned char row, unsigned char col)
{
    lcd_buffer[row / 8][col] &= ~(1 << (row % 8));
}

/*
 * Sets the pixel at (x, y) coordinates to the requested color.
 */
static void set_pixel_white(uint8_t x, uint8_t y)
{
    uint8_t yy = y;
    uint8_t row = yy / 8;
    uint8_t val = lcd_buffer[row][x];


    val |= (1 << (yy % 8));

    lcd_buffer[row][x] = val;
 
    /* Move LCD pointer to (x, y) coordinates */
    lcd_send(0x80 | x, LCD_CMD);    // column
    lcd_send(0x40 | row, LCD_CMD);  // row

    /* Set the pixel */
    lcd_send(val, LCD_DATA);

}

static void set_pixel_black(uint8_t x, uint8_t y)
{
    uint8_t yy = y;
    uint8_t row = yy / 8;
    uint8_t val = lcd_buffer[row][x];


    val &= ~(1 << (yy % 8));

    lcd_buffer[row][x] = val;
 
    /* Move LCD pointer to (x, y) coordinates */
    lcd_send(0x80 | x, LCD_CMD);    // column
    lcd_send(0x40 | row, LCD_CMD);  // row

    /* Set the pixel */
    lcd_send(val, LCD_DATA);
}

void draw_ball(int x, int y) {
	set_pixel_white(x-2,y);
	set_pixel_white(x-1,y-1);
	set_pixel_white(x-1,y);
	set_pixel_white(x-1,y+1);
	set_pixel_white(x,y-2);
	set_pixel_white(x,y-1);
	set_pixel_white(x,y);
	set_pixel_white(x,y+1);
	set_pixel_white(x,y+2);
	set_pixel_white(x+1,y-1);
	set_pixel_white(x+1,y);
	set_pixel_white(x+1,y+1);
	set_pixel_white(x+2,y);
}

void clear_ball(int x, int y) {
	set_pixel_black(x-2,y);
	set_pixel_black(x-1,y-1);
	set_pixel_black(x-1,y);
	set_pixel_black(x-1,y+1);
	set_pixel_black(x,y-2);
	set_pixel_black(x,y-1);
	set_pixel_black(x,y);
	set_pixel_black(x,y+1);
	set_pixel_black(x,y+2);
	set_pixel_black(x+1,y-1);
	set_pixel_black(x+1,y);
	set_pixel_black(x+1,y+1);
	set_pixel_black(x+2,y);
}

void draw_gap(int h, int x1, int x2) {
	for(int i = 1; i <= x1; i++) {
		set_pixel_white(i, h);
	}
	
	for(int i = x2; i <= LCD_X_RES; i++) {
		set_pixel_white(i, h);
	}
}


void draw_labyrinth_l1(void)
{
	int i = 7;
	for(int j = 0; j < 6; j++) {
		draw_gap(i, l1_gaps[j].x1, l1_gaps[j].x2);
		i += 7;
	}
}

void draw_labyrinth_l2(void)
{
	int i = 7;
	for(int j = 0; j < 6; j++) {
		draw_gap(i, l2_gaps[j].x1, l2_gaps[j].x2);
		i += 7;
	}
}

void draw_labyrinth_l3(void)
{
	int i = 7;
	for(int j = 0; j < 6; j++) {
		draw_gap(i, l3_gaps[j].x1, l3_gaps[j].x2);
		i += 7;
	}
}

void end_game_lost(void)
{

    lcd_clear();
     
    while(1)
    {
        // Print on first line
        lcd_goto_xy( 18, 1);
        lcd_str("Game Over");
    }
}

void end_game_win(void)
{

    lcd_clear();
     
    while(1)
    {
        // Print on first line
        lcd_goto_xy( 18, 1);
        lcd_str("Game won!:)");
    }
}

typedef struct Point2D
{
    int x, y;
} Point2D;


void init_io(void)
{
    DDRC = 0x00;
    PORTC = 0xFF;
	
	DDRD |= (1 << PD0);
	PORTD &= ~(1 << PD0);
    
    /* Disable JTAG port */ 
    MCUCR = _BV( JTD ); 
    MCUCR = _BV( JTD ); 
}

int main(void)
{   
	draw_world();
	ADC_init();

    int start = 0;
    int select = 0;
    int paused = 0;    
    
    // Setup LCD
    lcd_init();
    lcd_contrast(0x40);
    clear_lcd_buffer();
	draw_labyrinth_l1();
	
	init_io();
	
	draw_ball(xball, yball);
	
	int layer = 0;
	
	int time = 30000;
	
	while (1) {
		
		if(USE_ACCELEROMETER == 1) {
			int xval = ADC_get(YCHANNEL);

			if (xval - ACCY_0G > THRESHOLD){
				_delay_ms(1000);
				xball -= 1;
				xball = xball % LCD_X_RES;
			}
				
			else if (ACCY_0G - xval > THRESHOLD){
				_delay_ms(1000);
				xball += 1;
				xball = xball % LCD_X_RES;
			}
		}
		else{
			if (!(PINC & (1 << RIGHT))) {
			
				_delay_ms(1000);
				clear_ball(xball, yball);
				xball++;
				xball = xball % LCD_X_RES;
				
				draw_ball(xball, yball);
				time -= 100;
			}
		
			if(!(PINC & (1 << LEFT))) {
				_delay_ms(1000);
				clear_ball(xball, yball);
				
				xball--;
				if(xball < 0)
					xball = LCD_X_RES - 2;
				draw_ball(xball, yball);
				time -= 100;
			}
		}
		
		if(level == 1)
		if(xball >= l1_gaps[layer].x1 && xball <=l1_gaps[layer].x2 ) {
			layer++;
			if(layer == 6)
			{
				lcd_clear();
				clear_ball(xball, yball);
				draw_labyrinth_l2();
				level = 2;
				xball = 3;
				yball = 3;
				layer = 0;
				time = 30000;
				continue;
			}
			clear_ball(xball, yball);
			yball += 7;
			draw_ball(xball, yball);
		}
		
		if(level == 2)
		if(xball >= l2_gaps[layer].x1 && xball <=l2_gaps[layer].x2 ) {
			layer++;
			if(layer == 6)
			{
				lcd_clear();
				clear_ball(xball, yball);
				draw_labyrinth_l3();
				level = 3;
				xball = 3;
				yball = 3;
				layer = 0;
				time = 30000;
				continue;
			}
			clear_ball(xball, yball);
			yball += 7;
			draw_ball(xball, yball);
		}
		
		if(level == 3)
		if(xball >= l3_gaps[layer].x1 && xball <=l3_gaps[layer].x2 ) {
			layer++;
			if(layer == 6)
			{
				clear_lcd_buffer();
				PORTD |= (1 << PD0);
				_delay_ms(5000);
				PORTD &= ~(1 << PD0);
				end_game_win();
			}
			clear_ball(xball, yball);
			yball += 7;
			draw_ball(xball, yball);
		}
		
		time--;
		
		if(time <= 0) {
			clear_lcd_buffer();
			end_game_lost();
		}
		
		lcd_goto_xy(41, 20);
		char val[5];
		sprintf(val, "%d ", time/1000);
        lcd_str(val);
			
	}
    return 0;
}

